@@ -251,7 +251,7 @@ struct _snd_es1938 {
 
 static irqreturn_t snd_es1938_interrupt(int irq, void *dev_id, struct pt_regs *regs);
 
-static struct pci_device_id snd_es1938_ids[] __devinitdata = {
+static struct pci_device_id snd_es1938_ids[] = {
         { 0x125d, 0x1969, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0, },   /* Solo-1 */
 	{ 0, }
 };
@@ -1413,37 +1413,37 @@ static int __devinit snd_es1938_create(snd_card_t * card,
 	chip->pci = pci;
 	chip->io_port = pci_resource_start(pci, 0);
 	if ((chip->res_io_port = request_region(chip->io_port, 8, "ESS Solo-1")) == NULL) {
-		snd_es1938_free(chip);
 		snd_printk("unable to grab region 0x%lx-0x%lx\n", chip->io_port, chip->io_port + 8 - 1);
+		snd_es1938_free(chip);
 		return -EBUSY;
 	}
 	chip->sb_port = pci_resource_start(pci, 1);
 	if ((chip->res_sb_port = request_region(chip->sb_port, 0x10, "ESS Solo-1 SB")) == NULL) {
-		snd_es1938_free(chip);
 		snd_printk("unable to grab SB region 0x%lx-0x%lx\n", chip->sb_port, chip->sb_port + 0x10 - 1);
+		snd_es1938_free(chip);
 		return -EBUSY;
 	}
 	chip->vc_port = pci_resource_start(pci, 2);
 	if ((chip->res_vc_port = request_region(chip->vc_port, 0x10, "ESS Solo-1 VC (DMA)")) == NULL) {
-		snd_es1938_free(chip);
 		snd_printk("unable to grab VC (DMA) region 0x%lx-0x%lx\n", chip->vc_port, chip->vc_port + 0x10 - 1);
+		snd_es1938_free(chip);
 		return -EBUSY;
 	}
 	chip->mpu_port = pci_resource_start(pci, 3);
 	if ((chip->res_mpu_port = request_region(chip->mpu_port, 4, "ESS Solo-1 MIDI")) == NULL) {
-		snd_es1938_free(chip);
 		snd_printk("unable to grab MIDI region 0x%lx-0x%lx\n", chip->mpu_port, chip->mpu_port + 4 - 1);
+		snd_es1938_free(chip);
 		return -EBUSY;
 	}
 	chip->game_port = pci_resource_start(pci, 4);
 	if ((chip->res_game_port = request_region(chip->game_port, 4, "ESS Solo-1 GAME")) == NULL) {
-		snd_es1938_free(chip);
 		snd_printk("unable to grab GAME region 0x%lx-0x%lx\n", chip->game_port, chip->game_port + 4 - 1);
+		snd_es1938_free(chip);
 		return -EBUSY;
 	}
 	if (request_irq(pci->irq, snd_es1938_interrupt, SA_INTERRUPT|SA_SHIRQ, "ES1938", (void *)chip)) {
-		snd_es1938_free(chip);
 		snd_printk("unable to grab IRQ %d\n", pci->irq);
+		snd_es1938_free(chip);
 		return -EBUSY;
 	}
 	chip->irq = pci->irq;
