@@ -400,16 +400,13 @@ static void snd_us428_urb_capt_complete(purb_t urb, struct pt_regs *regs)
 		return;
 	}
 
-	if (pcm_captsubs)
-		if (pcm_captsubs->runtime->status->state == SNDRV_PCM_STATE_RUNNING)
-			runtime = pcm_captsubs->runtime;
+	if (pcm_captsubs && snd_pcm_running(pcm_captsubs))
+		runtime = pcm_captsubs->runtime;
 	if (NULL == runtime){
 		snd_us428_substream_t *playsubs = captsubs->stream->substream + SNDRV_PCM_STREAM_PLAYBACK;
 		snd_pcm_substream_t *pcm_playsubs = playsubs->pcm_substream;
-		if (pcm_playsubs)
-			if (pcm_playsubs->runtime->status->state == SNDRV_PCM_STATE_RUNNING)
-				runtime = pcm_playsubs->runtime;
-		
+		if (pcm_playsubs && snd_pcm_running(pcm_playsubs))
+			runtime = pcm_playsubs->runtime;
 	}
 		
 	if (NULL == runtime){
